import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

class AssetLibraryMeta:
    """AssetLibraryMeta(api_versions: dict[str, str], name: str, contact: Contact)"""

class Contact:
    """Contact(name: str, url: str | None = None, email: str | None = None)"""

    email: typing.Any
    url: typing.Any
