import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

class RawBlendFileReader:
    """Return a file handle to the raw blend file data (abstracting compressed formats)."""

def get_render_info_structure(endian_str, size) -> None: ...
def main() -> None: ...
def read_blend_rend_chunk(filepath) -> None: ...
